export interface IWarixArrayItemChange<T> {
    /** Array index of the item */
    index: number;
    /** Value of the item */
    item: T;
}
export interface IWarixArrayShiftChange<T> {
    /** Old index of the item in the Array */
    oldIndex: number;
    /** New index of the item in the array */
    newIndex: number;
    /** Value of the item */
    item: T;
}
export interface IWarixArrayOperationChange<T> {
    /** Old array value */
    oldValue: T[];
    /** New array value */
    newValue: T[];
    /** Added items to the array */
    addedItems: IWarixArrayItemChange<T>[];
    /** Removed items from the array */
    removedItems: IWarixArrayItemChange<T>[];
    /** Elements that changed position in the array */
    shiftedItems: IWarixArrayShiftChange<T>[];
}
export interface IWarixArrayGrouping<T> {
    /** Property name the group maps to */
    groupedBy: string;
    /** Property value the group maps to */
    groupedByValue: any;
    /** Zero based grouping level */
    groupLevel: number;
    /** Items or SubGroups of the group */
    items: T[] | IWarixArrayGrouping<T>[];
}
export interface IWarixArraySortDefinition {
    /** Property path */
    property: string;
    /** Descending? */
    descending: boolean;
}
export declare const arrayPush: <T>(baseArray: T[], ...items: T[]) => IWarixArrayOperationChange<T>;
export declare const arrayPop: <T>(baseArray: T[]) => IWarixArrayOperationChange<T>;
export declare const arrayShift: <T>(baseArray: T[]) => IWarixArrayOperationChange<T>;
export declare const arrayUnshift: <T>(baseArray: T[], ...items: T[]) => IWarixArrayOperationChange<T>;
export declare const arraySplice: <T>(baseArray: T[], start: number, deleteCount?: number, ...items: T[]) => IWarixArrayOperationChange<T>;
export declare const arrayInsert: <T>(baseArray: T[], index: number, ...items: T[]) => IWarixArrayOperationChange<T>;
export declare const arrayRemoveAt: <T>(baseArray: T[], index: number, removeCount?: number) => IWarixArrayOperationChange<T>;
export declare const arrayRemoveWhere: <T>(baseArray: T[], condition: (value: T, index?: number, source?: T[]) => boolean) => IWarixArrayOperationChange<T>;
export declare const arrayRemove: <T>(baseArray: T[], ...items: T[]) => IWarixArrayOperationChange<T>;
export declare const arraySort: <T>(baseArray: T[], compareFn?: (a: T, b: T) => number) => IWarixArrayOperationChange<T>;
export declare const arrayMultiSort: <T>(baseArray: T[], sorting: IWarixArraySortDefinition[]) => {
    oldValue: T[];
    newValue: T[];
    addedItems: any[];
    removedItems: any[];
    shiftedItems: {
        oldIndex: number;
        newIndex: number;
        item: T;
    }[];
};
export declare const arraySet: <T>(baseArray: T[], newValue: T[]) => IWarixArrayOperationChange<T>;
export declare const arraySetAt: <T>(baseArray: T[], index: number, value: T) => IWarixArrayOperationChange<T>;
export declare const arrayRevese: <T>(baseArray: T[]) => IWarixArrayOperationChange<T>;
export declare const arrayShuffle: <T>(baseArray: T[]) => IWarixArrayOperationChange<T>;
export declare const arrayDistinct: <T>(baseArray: T[]) => IWarixArrayOperationChange<T>;
export declare const arrayDistinctOf: <T>(baseArray: T[], propertyName: string) => any[];
export declare const arrayMove: <T>(baseArray: T[], fromIndex: number, toIndex: number) => IWarixArrayOperationChange<T>;
export declare const arrayGroupBy: <T>(baseArray: T[], ...groupByPropertiesNames: string[]) => IWarixArrayGrouping<T>[];
/**
 * Gets if the provided item complies witht the IWarixArrayGrouping interface
 * @param item Test item
 */
export declare const isGroupingItem: (item: any) => item is IWarixArrayGrouping<any>;
/**
 * Gets if the provided array complies witht the IWarixArrayGrouping interface for all its items
 * @param item Test array
 */
export declare const isGroupingArray: (array: any[]) => array is IWarixArrayGrouping<any>[];
