"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const fnNoAction = (baseArray) => {
    return {
        oldValue: baseArray,
        newValue: baseArray,
        addedItems: [],
        removedItems: [],
        shiftedItems: []
    };
};
const fnShuffle = (array) => {
    let counter = array.length;
    while (counter > 0) {
        const index = Math.floor(Math.random() * counter);
        counter--;
        const temp = array[counter];
        array[counter] = array[index];
        array[index] = temp;
    }
    return array;
};
exports.arrayPush = (baseArray, ...items) => {
    if (items.length === 0) {
        return fnNoAction(baseArray);
    }
    return {
        oldValue: baseArray,
        newValue: [...baseArray, ...items],
        addedItems: items.map((item, index) => ({ item, index: index + baseArray.length })),
        removedItems: [],
        shiftedItems: []
    };
};
exports.arrayPop = (baseArray) => {
    if (baseArray.length === 0) {
        return fnNoAction(baseArray);
    }
    const removed = { index: baseArray.length - 1, item: baseArray[baseArray.length - 1] };
    return {
        oldValue: baseArray,
        newValue: baseArray.slice(0, baseArray.length - 1),
        addedItems: [],
        removedItems: [removed],
        shiftedItems: []
    };
};
exports.arrayShift = (baseArray) => {
    if (baseArray.length === 0) {
        return fnNoAction(baseArray);
    }
    const removed = { index: 0, item: baseArray[0] };
    const newValue = baseArray.slice(1);
    return {
        oldValue: baseArray,
        newValue,
        addedItems: [],
        removedItems: [removed],
        shiftedItems: newValue.map((item, i) => ({ oldIndex: i - 1, newIndex: i, item }))
    };
};
exports.arrayUnshift = (baseArray, ...items) => {
    if (items.length === 0) {
        return fnNoAction(baseArray);
    }
    const newValue = [...items, ...baseArray];
    return {
        oldValue: baseArray,
        newValue,
        addedItems: items.map((item, index) => ({ item, index })),
        removedItems: [],
        shiftedItems: baseArray.map((item, oldIndex) => ({ oldIndex, newIndex: oldIndex + items.length, item }))
    };
};
exports.arraySplice = (baseArray, start, deleteCount = 0, ...items) => {
    const mappedClone = baseArray.slice(0).map((item, index) => ({ oldIndex: index, newIndex: index, item }));
    const removedClone = mappedClone.splice(start, deleteCount, ...items.map(item => ({ oldIndex: -1, newIndex: -1, item })));
    mappedClone.forEach((item, index) => item.newIndex = index);
    return {
        oldValue: baseArray,
        newValue: mappedClone.map(x => x.item),
        addedItems: mappedClone.filter(x => x.oldIndex === -1).map(x => ({ index: x.newIndex, item: x.item })),
        removedItems: removedClone.map(x => ({ index: x.oldIndex, item: x.item })),
        shiftedItems: mappedClone.filter(x => x.oldIndex !== x.newIndex && x.oldIndex > -1)
    };
};
exports.arrayInsert = (baseArray, index, ...items) => {
    if (items.length === 0) {
        return fnNoAction(baseArray);
    }
    return exports.arraySplice(baseArray, index, 0, ...items);
};
exports.arrayRemoveAt = (baseArray, index, removeCount = 1) => {
    if (removeCount <= 0) {
        return fnNoAction(baseArray);
    }
    return exports.arraySplice(baseArray, index, removeCount);
};
exports.arrayRemoveWhere = (baseArray, condition) => {
    const allMapped = baseArray.map((item, oldIndex) => ({ oldIndex, newIndex: oldIndex, item, remove: condition(item, oldIndex, baseArray) }));
    const removedItems = allMapped.filter(x => x.remove).map(x => ({ item: x.item, index: x.oldIndex }));
    const result = allMapped.filter(x => x.remove === false);
    result.forEach((x, index) => x.newIndex = index);
    return {
        oldValue: baseArray,
        newValue: result.map(x => x.item),
        addedItems: [],
        removedItems,
        shiftedItems: allMapped.filter(x => x.oldIndex !== x.newIndex).map(x => ({ oldIndex: x.oldIndex, newIndex: x.newIndex, item: x.item }))
    };
};
exports.arrayRemove = (baseArray, ...items) => exports.arrayRemoveWhere(baseArray, i => items.indexOf(i) > -1);
exports.arraySort = (baseArray, compareFn) => {
    const mapped = baseArray.map((item, oldIndex) => ({ oldIndex, newIndex: oldIndex, item })).sort((a, b) => {
        return compareFn ? compareFn(a.item, b.item) : (a.item < b.item ? -1 : a.item === b.item ? 0 : 1);
    });
    return {
        oldValue: baseArray,
        newValue: mapped.map(x => x.item),
        addedItems: [],
        removedItems: [],
        shiftedItems: mapped.filter(x => x.newIndex !== x.oldIndex)
    };
};
exports.arrayMultiSort = (baseArray, sorting) => {
    const fnApplySort = (a, b, c, n) => {
        const ax = a[c.property];
        const bx = b[c.property];
        const mx = c.descending ? -1 : 1;
        return ax < bx ? (-1 * mx) : bx < ax ? (1 * mx) : (n.length > 0 ? fnApplySort(a, b, n[0], n.slice(1)) : 0);
    };
    const mapped = baseArray.map((item, oldIndex) => ({ oldIndex, newIndex: oldIndex, item })).sort((a, b) => {
        return fnApplySort(a.item, b.item, sorting[0], sorting.slice(1));
    });
    return {
        oldValue: baseArray,
        newValue: mapped.map(x => x.item),
        addedItems: [],
        removedItems: [],
        shiftedItems: mapped.filter(x => x.newIndex !== x.oldIndex)
    };
};
exports.arraySet = (baseArray, newValue) => {
    return {
        oldValue: baseArray,
        newValue,
        addedItems: newValue.map((item, index) => ({ item, index })),
        removedItems: baseArray.map((item, index) => ({ item, index })),
        shiftedItems: []
    };
};
exports.arraySetAt = (baseArray, index, value) => exports.arraySplice(baseArray, index, 1, value);
exports.arrayRevese = (baseArray) => {
    const newValue = baseArray.map((item, index) => ({ oldIndex: index, newIndex: index, item })).reverse();
    newValue.forEach((x, i) => x.newIndex = i);
    return {
        oldValue: baseArray,
        newValue: newValue.map(x => x.item),
        addedItems: [],
        removedItems: [],
        shiftedItems: newValue.filter(x => x.oldIndex !== x.newIndex)
    };
};
exports.arrayShuffle = (baseArray) => {
    const mapped = fnShuffle(baseArray.map((item, oldIndex) => ({ oldIndex, newIndex: oldIndex, item })));
    mapped.forEach((x, index) => x.newIndex = index);
    return {
        oldValue: baseArray,
        newValue: mapped.map(x => x.item),
        addedItems: [],
        removedItems: [],
        shiftedItems: mapped.filter(x => x.newIndex !== x.oldIndex)
    };
};
exports.arrayDistinct = (baseArray) => exports.arrayRemoveWhere(baseArray, (value, index, array) => array.indexOf(value) !== index);
exports.arrayDistinctOf = (baseArray, propertyName) => baseArray.map(x => x[propertyName]).filter((xx, ii, aa) => aa.indexOf(xx) === ii);
exports.arrayMove = (baseArray, fromIndex, toIndex) => exports.arraySplice(baseArray, toIndex, 0, baseArray.slice(0).splice(fromIndex, 1)[0]);
exports.arrayGroupBy = (baseArray, ...groupByPropertiesNames) => {
    const fnApplyGrouping = (ax, prop, nextProps, level = 0) => {
        return exports.arrayDistinct(ax.map(i => i[prop])).newValue.map(distinctValueEntry => {
            const applicableItems = ax.filter(i => i[prop] === distinctValueEntry);
            const ret = {
                groupedBy: prop,
                groupedByValue: distinctValueEntry,
                groupLevel: level,
                items: nextProps.length === 0 ? applicableItems : fnApplyGrouping(applicableItems, nextProps[0], nextProps.slice(1), level + 1)
            };
            return ret;
        });
    };
    return (baseArray || []).length === 0 || (groupByPropertiesNames || []).length === 0 ?
        [{ groupedBy: null, groupedByValue: null, items: baseArray, groupLevel: 0 }] :
        fnApplyGrouping(baseArray, groupByPropertiesNames[0], groupByPropertiesNames.slice(1), 0);
};
/**
 * Gets if the provided item complies witht the IWarixArrayGrouping interface
 * @param item Test item
 */
exports.isGroupingItem = (item) => {
    return item && lodash_1.isObjectLike(item) && 'groupedBy' in item && 'groupedByValue' in item && 'groupLevel' in item && 'items' in item;
};
/**
 * Gets if the provided array complies witht the IWarixArrayGrouping interface for all its items
 * @param item Test array
 */
exports.isGroupingArray = (array) => {
    return array && Array.isArray(array) && array.every(x => exports.isGroupingItem(x));
};
//# sourceMappingURL=warix.array-operations.js.map