export declare type WarixDataFilter = (data: any) => boolean;
export declare type WarixDataFilterDefinition = (propertyPath: string, referenceValue: any) => WarixDataFilter;
export declare type WarixDataFilterGroupJoinType = 'and' | 'or';
export declare enum WarixDataFilterGroupJoin {
    AND = "and",
    OR = "or"
}
export interface IWarixDataFilterEntry {
    propertyPath: string;
    operation: string;
    refValue: any;
}
export interface IWarixDataFilterGroup {
    type: WarixDataFilterGroupJoin | WarixDataFilterGroupJoinType;
    operations: (IWarixDataFilterEntry | IWarixDataFilterGroup)[];
}
export interface IWarixDataFilterProvider {
    [key: string]: WarixDataFilterDefinition;
}
export declare const WarixDataFilters: {
    isNil: (propertyPath: string, ref: any) => (data: any) => boolean;
    nil: (propertyPath: string, ref: any) => (data: any) => boolean;
    isNotNil: (propertyPath: string, ref: any) => (data: any) => boolean;
    nnil: (propertyPath: string, ref: any) => (data: any) => boolean;
    equals: (propertyPath: string, ref: any) => (data: any) => boolean;
    eq: (propertyPath: string, ref: any) => (data: any) => boolean;
    notEquals: (propertyPath: string, ref: any) => (data: any) => boolean;
    neq: (propertyPath: string, ref: any) => (data: any) => boolean;
    lowerThan: (propertyPath: string, ref: any) => (data: any) => boolean;
    lt: (propertyPath: string, ref: any) => (data: any) => boolean;
    lowerOrEqualTo: (propertyPath: string, ref: any) => (data: any) => boolean;
    lte: (propertyPath: string, ref: any) => (data: any) => boolean;
    greaterThan: (propertyPath: string, ref: any) => (data: any) => boolean;
    gt: (propertyPath: string, ref: any) => (data: any) => boolean;
    greaterOrEqualTo: (propertyPath: string, ref: any) => (data: any) => boolean;
    gte: (propertyPath: string, ref: any) => (data: any) => boolean;
    contains: (propertyPath: string, ref: any) => (data: any) => boolean;
    cc: (propertyPath: string, ref: any) => (data: any) => boolean;
    startsWith: (propertyPath: string, ref: any) => (data: any) => boolean;
    sw: (propertyPath: string, ref: any) => (data: any) => boolean;
    endsWith: (propertyPath: string, ref: any) => (data: any) => boolean;
    ew: (propertyPath: string, ref: any) => (data: any) => boolean;
    isIn: (propertyPath: string, possibleValues: any[]) => (data: any) => boolean;
    in: (propertyPath: string, possibleValues: any[]) => (data: any) => boolean;
    isNotIn: (propertyPath: string, possibleValues: any[]) => (data: any) => boolean;
    nin: (propertyPath: string, possibleValues: any[]) => (data: any) => boolean;
};
export declare const AND: (...operations: WarixDataFilter[]) => (dataOwner: any) => boolean;
export declare const OR: (...operations: WarixDataFilter[]) => (dataOwner: any) => boolean;
export declare function compileGroup(group: IWarixDataFilterGroup, ...providers: IWarixDataFilterProvider[]): (dataOwner: any) => boolean;
export declare function groupContainsProperty(group: IWarixDataFilterGroup, propertyPath: string): boolean;
export declare function groupProperties(group: IWarixDataFilterGroup): string[];
