"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.bounds = (target) => {
    const b = target.getBoundingClientRect();
    return {
        x: b.left,
        y: b.top,
        left: b.left,
        top: b.top,
        right: b.right,
        bottom: b.bottom,
        cx: b.left + (b.width * .5),
        cy: b.top + (b.height * .5),
        width: b.width,
        height: b.height
    };
};
exports.offsetPosition = (e) => {
    const coords = { x: 0, y: 0 };
    while (e) {
        coords.x += e.offsetLeft;
        coords.y += e.offsetTop;
        e = e.offsetParent;
    }
    return coords;
};
exports.offsetSize = (e) => {
    return { width: e.offsetWidth, height: e.offsetHeight };
};
exports.offsetBounds = (e) => {
    const p = exports.offsetPosition(e);
    const s = exports.offsetSize(e);
    return Object.assign({}, p, s, {
        left: p.x,
        top: p.y,
        right: p.x + s.width,
        bottom: p.y + s.height,
        cx: p.x + (s.width * .5),
        cy: p.y + (s.height * .5)
    });
};
exports.mouseCoordsToClient = (client, evt) => {
    const bb = exports.bounds(client);
    return {
        x: evt.clientX - bb.left,
        y: evt.clientY - bb.top
    };
};
exports.scrollbarDimensions = (element) => {
    const b = exports.bounds(element);
    return {
        width: Math.ceil(b.width - element.scrollWidth),
        height: Math.ceil(b.height - element.scrollHeight)
    };
};
exports.scrollbarVisibility = (element) => {
    return {
        left: element.scrollWidth > element.clientWidth,
        top: element.scrollHeight > element.clientHeight
    };
};
exports.scrollPercent = (element) => {
    return {
        left: element.scrollLeft / (element.scrollWidth - element.clientWidth),
        top: element.scrollTop / (element.scrollHeight - element.clientHeight)
    };
};
//# sourceMappingURL=warix.dom.js.map